@@ -57,6 +57,10 @@ typedef struct
     PyObject *obj;
 } PyGRealSource;
 
+/* glib.Source */
+
+PYGLIB_DEFINE_TYPE("glib.Source", PyGSource_Type, PyGSource)
+
 static PyObject *
 source_repr(PyGSource *self, const char *type)
 {
@@ -78,7 +82,7 @@ source_repr(PyGSource *self, const char *type)
 	g_snprintf(buf, sizeof(buf), "<%s glib source at 0x%lx>",
 		   desc, (long) self);
 
-    return PyString_FromString(buf);
+    return _PyUnicode_FromString(buf);
 }
 
 static PyObject *
@@ -105,7 +109,7 @@ pyg_source_attach(PyGSource *self, PyObject *args, PyObject *kwargs)
     }
 
     id = g_source_attach(self->source, context);
-    return PyInt_FromLong(id);
+    return _PyLong_FromLong(id);
 }
 
 static PyObject *
@@ -279,7 +283,7 @@ pyg_source_get_priority(PyGSource *self, void *closure)
 {
     CHECK_DESTROYED(self, NULL);
 
-    return PyInt_FromLong(g_source_get_priority(self->source));
+    return _PyLong_FromLong(g_source_get_priority(self->source));
 }
 
 static int
@@ -292,12 +296,12 @@ pyg_source_set_priority(PyGSource *self, PyObject *value, void *closure)
 	return -1;
     }
 
-    if (!PyInt_Check(value)) {
+    if (!_PyLong_Check(value)) {
 	PyErr_SetString(PyExc_TypeError, "type mismatch");
 	return -1;
     }
 
-    g_source_set_priority(self->source, PyInt_AsLong(value));
+    g_source_set_priority(self->source, _PyLong_AsLong(value));
 
     return 0;
 }
@@ -335,7 +339,7 @@ pyg_source_get_id(PyGSource *self, void *closure)
 	return NULL;
     }
 
-    return PyInt_FromLong(g_source_get_id(self->source));
+    return _PyLong_FromLong(g_source_get_id(self->source));
 }
 
 static PyGetSetDef pyg_source_getsets[] = {
@@ -422,7 +426,7 @@ pyg_source_prepare(GSource *source, gint *timeout)
     }
 
     ret = PyObject_IsTrue(PyTuple_GET_ITEM(t, 0));
-	*timeout = PyInt_AsLong(PyTuple_GET_ITEM(t, 1));
+	*timeout = _PyLong_AsLong(PyTuple_GET_ITEM(t, 1));
 
 	if (*timeout == -1 && PyErr_Occurred()) {
 	    ret = FALSE;
@@ -558,51 +562,9 @@ pyg_source_free(PyObject *op)
     PyObject_GC_Del(op);
 }
 
-PyTypeObject PyGSource_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,					/* ob_size */
-    "gobject.Source",			/* tp_name */
-    sizeof(PyGSource),			/* tp_basicsize */
-    0,					/* tp_itemsize */
-    (destructor)pyg_source_dealloc,	/* tp_dealloc */
-    (printfunc)0,			/* tp_print */
-    (getattrfunc)0,			/* tp_getattr */
-    (setattrfunc)0,			/* tp_setattr */
-    (cmpfunc)0,				/* tp_compare */
-    (reprfunc)pyg_source_repr,		/* tp_repr */
-    0,					/* tp_as_number */
-    0,					/* tp_as_sequence */
-    0,					/* tp_as_mapping */
-    (hashfunc)0,			/* tp_hash */
-    (ternaryfunc)0,			/* tp_call */
-    (reprfunc)0,			/* tp_str */
-    (getattrofunc)0,			/* tp_getattro */
-    (setattrofunc)0,			/* tp_setattro */
-    0,					/* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE |
-      Py_TPFLAGS_HAVE_GC,		/* tp_flags */
-    NULL,				/* tp_doc */
-    (traverseproc)pyg_source_traverse,	/* tp_traverse */
-    (inquiry)pyg_source_clear,		/* tp_clear */
-    (richcmpfunc)0,			/* tp_richcompare */
-    offsetof(PyGSource, weakreflist),	/* tp_weaklistoffset */
-    (getiterfunc)0,			/* tp_iter */
-    (iternextfunc)0,			/* tp_iternext */
-    pyg_source_methods,			/* tp_methods */
-    0,					/* tp_members */
-    pyg_source_getsets,			/* tp_getset */
-    (PyTypeObject *)0,			/* tp_base */
-    (PyObject *)0,			/* tp_dict */
-    0,					/* tp_descr_get */
-    0,					/* tp_descr_set */
-    offsetof(PyGSource, inst_dict),	/* tp_dictoffset */
-    (initproc)pyg_source_init,		/* tp_init */
-    (allocfunc)0,			/* tp_alloc */
-    (newfunc)0,				/* tp_new */
-    (freefunc)pyg_source_free,		/* tp_free */
-    (inquiry)0,				/* tp_is_gc */
-    (PyObject *)0,			/* tp_bases */
-};
+/* glib.Idle */
+
+PYGLIB_DEFINE_TYPE("glib.Idle", PyGIdle_Type, PyGSource)
 
 static PyObject *
 pyg_idle_repr(PyGSource *self)
@@ -617,7 +579,7 @@ pyg_idle_init(PyGSource *self, PyObject *args, PyObject *kwargs)
     gint priority = G_PRIORITY_DEFAULT_IDLE;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-				     "|i:gobject.Idle.__init__", kwlist,
+				     "|i:glib.Idle.__init__", kwlist,
 				     &priority))
 	return -1;
 
@@ -634,50 +596,9 @@ pyg_idle_init(PyGSource *self, PyObject *args, PyObject *kwargs)
     return 0;
 }
 
-PyTypeObject PyGIdle_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,					/* ob_size */
-    "gobject.Idle",			/* tp_name */
-    sizeof(PyGSource),			/* tp_basicsize */
-    0,					/* tp_itemsize */
-    (destructor)0,			/* tp_dealloc */
-    (printfunc)0,			/* tp_print */
-    (getattrfunc)0,			/* tp_getattr */
-    (setattrfunc)0,			/* tp_setattr */
-    (cmpfunc)0,				/* tp_compare */
-    (reprfunc)pyg_idle_repr,		/* tp_repr */
-    0,					/* tp_as_number */
-    0,					/* tp_as_sequence */
-    0,					/* tp_as_mapping */
-    (hashfunc)0,			/* tp_hash */
-    (ternaryfunc)0,			/* tp_call */
-    (reprfunc)0,			/* tp_str */
-    (getattrofunc)0,			/* tp_getattro */
-    (setattrofunc)0,			/* tp_setattro */
-    0,					/* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,		/* tp_flags */
-    NULL,				/* tp_doc */
-    (traverseproc)0,			/* tp_traverse */
-    (inquiry)0,				/* tp_clear */
-    (richcmpfunc)0,			/* tp_richcompare */
-    0,					/* tp_weaklistoffset */
-    (getiterfunc)0,			/* tp_iter */
-    (iternextfunc)0,			/* tp_iternext */
-    NULL,				/* tp_methods */
-    NULL,				/* tp_members */
-    NULL,				/* tp_getset */
-    (PyTypeObject *)&PyGSource_Type,	/* tp_base */
-    (PyObject *)0,			/* tp_dict */
-    0,					/* tp_descr_get */
-    0,					/* tp_descr_set */
-    0,					/* tp_dictoffset */
-    (initproc)pyg_idle_init,		/* tp_init */
-    (allocfunc)0,			/* tp_alloc */
-    (newfunc)0,				/* tp_new */
-    (freefunc)0,			/* tp_free */
-    (inquiry)0,				/* tp_is_gc */
-    (PyObject *)0,			/* tp_bases */
-};
+/* glib.Timeout */
+
+PYGLIB_DEFINE_TYPE("glib.Timeout", PyGTimeout_Type, PyGSource)
 
 static PyObject *
 pyg_timeout_repr(PyGSource *self)
@@ -693,7 +614,7 @@ pyg_timeout_init(PyGSource *self, PyObject *args, PyObject *kwargs)
     guint interval;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-				     "I|i:gobject.Timeout.__init__", kwlist,
+				     "I|i:glib.Timeout.__init__", kwlist,
 				     &interval, &priority))
 	return -1;
 
@@ -710,55 +631,14 @@ pyg_timeout_init(PyGSource *self, PyObject *args, PyObject *kwargs)
     return 0;
 }
 
-PyTypeObject PyGTimeout_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,					/* ob_size */
-    "gobject.Timeout",			/* tp_name */
-    sizeof(PyGSource),			/* tp_basicsize */
-    0,					/* tp_itemsize */
-    (destructor)0,			/* tp_dealloc */
-    (printfunc)0,			/* tp_print */
-    (getattrfunc)0,			/* tp_getattr */
-    (setattrfunc)0,			/* tp_setattr */
-    (cmpfunc)0,				/* tp_compare */
-    (reprfunc)pyg_timeout_repr,		/* tp_repr */
-    0,					/* tp_as_number */
-    0,					/* tp_as_sequence */
-    0,					/* tp_as_mapping */
-    (hashfunc)0,			/* tp_hash */
-    (ternaryfunc)0,			/* tp_call */
-    (reprfunc)0,			/* tp_str */
-    (getattrofunc)0,			/* tp_getattro */
-    (setattrofunc)0,			/* tp_setattro */
-    0,					/* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,		/* tp_flags */
-    NULL,				/* tp_doc */
-    (traverseproc)0,			/* tp_traverse */
-    (inquiry)0,				/* tp_clear */
-    (richcmpfunc)0,			/* tp_richcompare */
-    0,					/* tp_weaklistoffset */
-    (getiterfunc)0,			/* tp_iter */
-    (iternextfunc)0,			/* tp_iternext */
-    NULL,				/* tp_methods */
-    NULL,				/* tp_members */
-    NULL,				/* tp_getset */
-    (PyTypeObject *)&PyGSource_Type,	/* tp_base */
-    (PyObject *)0,			/* tp_dict */
-    0,					/* tp_descr_get */
-    0,					/* tp_descr_set */
-    0,					/* tp_dictoffset */
-    (initproc)pyg_timeout_init,		/* tp_init */
-    (allocfunc)0,			/* tp_alloc */
-    (newfunc)0,				/* tp_new */
-    (freefunc)0,			/* tp_free */
-    (inquiry)0,				/* tp_is_gc */
-    (PyObject *)0,			/* tp_bases */
-};
+/* glib.PollFD */
+
+PYGLIB_DEFINE_TYPE("glib.PollFD", PyGPollFD_Type, PyGPollFD)
 
 static PyMemberDef pyg_poll_fd_members[] = {
-    { "fd",      T_INT,    offsetof(PyGPollFD, pollfd.fd),      RO },
-    { "events",  T_USHORT, offsetof(PyGPollFD, pollfd.events),  RO },
-    { "revents", T_USHORT, offsetof(PyGPollFD, pollfd.revents), RO },
+    { "fd",      T_INT,    offsetof(PyGPollFD, pollfd.fd),      READONLY },
+    { "events",  T_USHORT, offsetof(PyGPollFD, pollfd.events),  READONLY },
+    { "revents", T_USHORT, offsetof(PyGPollFD, pollfd.revents), READONLY },
     { NULL, 0, 0, 0 }
 };
 
@@ -772,9 +652,9 @@ pyg_poll_fd_dealloc(PyGPollFD *self)
 static PyObject *
 pyg_poll_fd_repr(PyGPollFD *self)
 {
-    return PyString_FromFormat("<GPollFD %d (%d) at 0x%lx>",
-			       self->pollfd.fd, self->pollfd.events,
-			       (long)self);
+    return _PyUnicode_FromFormat("<GPollFD %d (%d) at 0x%lx>",
+				 self->pollfd.fd, self->pollfd.events,
+				 (long)self);
 }
 
 static int
@@ -786,7 +666,7 @@ pyg_poll_fd_init(PyGPollFD *self, PyObject *args, PyObject *kwargs)
     gushort events;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-				     "OH:gobject.PollFD.__init__", kwlist,
+				     "OH:glib.PollFD.__init__", kwlist,
 				     &o, &events))
 	return -1;
 
@@ -804,56 +684,40 @@ pyg_poll_fd_init(PyGPollFD *self, PyObject *args, PyObject *kwargs)
     return 0;
 }
 
-PyTypeObject PyGPollFD_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,					/* ob_size */
-    "gobject.PollFD",			/* tp_name */
-    sizeof(PyGPollFD),			/* tp_basicsize */
-    0,					/* tp_itemsize */
-    (destructor)pyg_poll_fd_dealloc,	/* tp_dealloc */
-    (printfunc)0,			/* tp_print */
-    (getattrfunc)0,			/* tp_getattr */
-    (setattrfunc)0,			/* tp_setattr */
-    (cmpfunc)0,				/* tp_compare */
-    (reprfunc)pyg_poll_fd_repr,		/* tp_repr */
-    0,					/* tp_as_number */
-    0,					/* tp_as_sequence */
-    0,					/* tp_as_mapping */
-    (hashfunc)0,			/* tp_hash */
-    (ternaryfunc)0,			/* tp_call */
-    (reprfunc)0,			/* tp_str */
-    (getattrofunc)0,			/* tp_getattro */
-    (setattrofunc)0,			/* tp_setattro */
-    0,					/* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,			/* tp_flags */
-    NULL,				/* tp_doc */
-    (traverseproc)0,			/* tp_traverse */
-    (inquiry)0,				/* tp_clear */
-    (richcmpfunc)0,			/* tp_richcompare */
-    0,					/* tp_weaklistoffset */
-    (getiterfunc)0,			/* tp_iter */
-    (iternextfunc)0,			/* tp_iternext */
-    0,					/* tp_methods */
-    pyg_poll_fd_members,		/* tp_members */
-    0,					/* tp_getset */
-    (PyTypeObject *)0,			/* tp_base */
-    (PyObject *)0,			/* tp_dict */
-    0,					/* tp_descr_get */
-    0,					/* tp_descr_set */
-    0,					/* tp_dictoffset */
-    (initproc)pyg_poll_fd_init,		/* tp_init */
-    (allocfunc)0,			/* tp_alloc */
-    (newfunc)0,				/* tp_new */
-    (freefunc)0,			/* tp_free */
-    (inquiry)0,				/* tp_is_gc */
-    (PyObject *)0,			/* tp_bases */
-};
-
 void
 pyglib_source_register_types(PyObject *d)
 {
+    PyGSource_Type.tp_flags = (Py_TPFLAGS_DEFAULT |
+			       Py_TPFLAGS_BASETYPE |
+			       Py_TPFLAGS_HAVE_GC);
+    PyGSource_Type.tp_init = (initproc)pyg_source_init;
+    PyGSource_Type.tp_free = (freefunc)pyg_source_free;
+    PyGSource_Type.tp_dealloc = (destructor)pyg_source_dealloc;
+    PyGSource_Type.tp_methods = pyg_source_methods;
+    PyGSource_Type.tp_repr = (reprfunc)pyg_source_repr;
+    PyGSource_Type.tp_traverse = (traverseproc)pyg_source_traverse;
+    PyGSource_Type.tp_clear = (inquiry)pyg_source_clear;
+    PyGSource_Type.tp_getset = pyg_source_getsets;
+    PyGSource_Type.tp_weaklistoffset = offsetof(PyGSource, weakreflist);
+    PyGSource_Type.tp_dictoffset = offsetof(PyGSource, inst_dict);
     PYGLIB_REGISTER_TYPE(d, PyGSource_Type, "Source");
+
+    PyGIdle_Type.tp_repr = (reprfunc)pyg_idle_repr;
+    PyGIdle_Type.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE;
+    PyGIdle_Type.tp_base = (PyTypeObject *)&PyGSource_Type;
+    PyGIdle_Type.tp_init = (initproc)pyg_idle_init;
     PYGLIB_REGISTER_TYPE(d, PyGIdle_Type, "Idle");
+
+    PyGTimeout_Type.tp_repr = (reprfunc)pyg_timeout_repr;
+    PyGTimeout_Type.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE;
+    PyGTimeout_Type.tp_base = (PyTypeObject *)&PyGSource_Type;
+    PyGTimeout_Type.tp_init = (initproc)pyg_timeout_init;
     PYGLIB_REGISTER_TYPE(d, PyGTimeout_Type, "Timeout");
+
+    PyGPollFD_Type.tp_dealloc = (destructor)pyg_poll_fd_dealloc;
+    PyGPollFD_Type.tp_repr = (reprfunc)pyg_poll_fd_repr;
+    PyGPollFD_Type.tp_flags = Py_TPFLAGS_DEFAULT;
+    PyGPollFD_Type.tp_members = pyg_poll_fd_members;
+    PyGPollFD_Type.tp_init = (initproc)pyg_poll_fd_init;
     PYGLIB_REGISTER_TYPE(d, PyGPollFD_Type, "PollFD");
 }
