@@ -2,7 +2,7 @@
 # pygobject - Python bindings for the GObject library
 # Copyright (C) 2007 Johan Dahlin
 #
-#   gobject/_property.py: GObject property wrapper/helper
+#   gobject/propertyhelper.py: GObject property wrapper/helper
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,11 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 # USA
 
-import _gobject
+import sys
+
+import gobject._gobject
+_gobject = sys.modules['gobject._gobject']
+
 from gobject.constants import \
      TYPE_NONE, TYPE_INTERFACE, TYPE_CHAR, TYPE_UCHAR, \
      TYPE_BOOLEAN, TYPE_INT, TYPE_UINT, TYPE_LONG, \
@@ -33,15 +37,10 @@ from gobject.constants import \
      G_MAXULONG
 
 
-class PropertyMeta(type):
-    def __repr__(self):
-        return "<class 'gobject.property'>"
-
-
 class property(object):
     """
-    Creates a new property which in conjunction with GObjectMeta will
-    create a GObject property proxy:
+    Creates a new property which in conjunction with GObject subclass will
+    create a property proxy:
 
     >>> class MyObject(gobject.GObject):
     >>> ... prop = gobject.property(type=str)
@@ -51,9 +50,20 @@ class property(object):
 
     >>> obj.prop
     'value'
+
+    The API is similar to the builtin property:
+
+    class AnotherObject(gobject.GObject):
+        @gobject.property
+        def prop(self):
+            return ...
+
+    Which will create a read-only property called prop.
     """
 
-    __metaclass__ = PropertyMeta
+    class __metaclass__(type):
+        def __repr__(self):
+            return "<class 'gobject.property'>"
 
     def __init__(self, getter=None, setter=None, type=None, default=None,
                  nick='', blurb='', flags=_gobject.PARAM_READWRITE,
@@ -203,9 +213,7 @@ class property(object):
     def _check_default(self):
         ptype = self.type
         default = self.default
-        if (ptype == TYPE_BOOLEAN and
-            (default is not True or
-             default is not False)):
+        if (ptype == TYPE_BOOLEAN and (default not in (True, False))):
             raise TypeError(
                 "default must be True or False, not %r" % (default,))
         elif ptype == TYPE_PYOBJECT:
