@@ -24,18 +24,25 @@
 #  include <config.h>
 #endif
 
+#include <pyglib.h>
 #include "pygobject-private.h"
+#include "pygboxed.h"
+
+GQuark pygboxed_type_key;
+GQuark pygboxed_marshal_key;
+
+PYGLIB_DEFINE_TYPE("gobject.GBoxed", PyGBoxed_Type, PyGBoxed);
 
 static void
 pyg_boxed_dealloc(PyGBoxed *self)
 {
     if (self->free_on_dealloc && self->boxed) {
-	PyGILState_STATE state = pyg_gil_state_ensure();
+	PyGILState_STATE state = pyglib_gil_state_ensure();
 	g_boxed_free(self->gtype, self->boxed);
-	pyg_gil_state_release(state);
+	pyglib_gil_state_release(state);
     }
 
-    self->ob_type->tp_free((PyObject *)self);
+    Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
 static int
@@ -59,7 +66,7 @@ pyg_boxed_repr(PyGBoxed *self)
 
     g_snprintf(buf, sizeof(buf), "<%s at 0x%lx>", g_type_name(self->gtype),
 	       (long)self->boxed);
-    return PyString_FromString(buf);
+    return _PyUnicode_FromString(buf);
 }
 
 static int
@@ -74,7 +81,8 @@ pyg_boxed_init(PyGBoxed *self, PyObject *args, PyObject *kwargs)
     self->gtype = 0;
     self->free_on_dealloc = FALSE;
 
-    g_snprintf(buf, sizeof(buf), "%s can not be constructed", self->ob_type->tp_name);
+    g_snprintf(buf, sizeof(buf), "%s can not be constructed",
+	       Py_TYPE(self)->tp_name);
     PyErr_SetString(PyExc_NotImplementedError, buf);
     return -1;
 }
@@ -99,54 +107,6 @@ static PyMethodDef pygboxed_methods[] = {
 };
 
 
-
-PyTypeObject PyGBoxed_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,                                  /* ob_size */
-    "gobject.GBoxed",                   /* tp_name */
-    sizeof(PyGBoxed),                   /* tp_basicsize */
-    0,                                  /* tp_itemsize */
-    /* methods */
-    (destructor)pyg_boxed_dealloc,      /* tp_dealloc */
-    (printfunc)0,                       /* tp_print */
-    (getattrfunc)0,                     /* tp_getattr */
-    (setattrfunc)0,                     /* tp_setattr */
-    (cmpfunc)pyg_boxed_compare,         /* tp_compare */
-    (reprfunc)pyg_boxed_repr,           /* tp_repr */
-    0,                                  /* tp_as_number */
-    0,                                  /* tp_as_sequence */
-    0,                                  /* tp_as_mapping */
-    (hashfunc)pyg_boxed_hash,           /* tp_hash */
-    (ternaryfunc)0,                     /* tp_call */
-    (reprfunc)0,                        /* tp_str */
-    (getattrofunc)0,			/* tp_getattro */
-    (setattrofunc)0,                    /* tp_setattro */
-    0,					/* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/* tp_flags */
-    NULL, /* Documentation string */
-    (traverseproc)0,			/* tp_traverse */
-    (inquiry)0,				/* tp_clear */
-    (richcmpfunc)0,			/* tp_richcompare */
-    0,					/* tp_weaklistoffset */
-    (getiterfunc)0,			/* tp_iter */
-    (iternextfunc)0,			/* tp_iternext */
-    pygboxed_methods,                   /* tp_methods */
-    0,					/* tp_members */
-    0,					/* tp_getset */
-    (PyTypeObject *)0,			/* tp_base */
-    (PyObject *)0,			/* tp_dict */
-    0,					/* tp_descr_get */
-    0,					/* tp_descr_set */
-    0,					/* tp_dictoffset */
-    (initproc)pyg_boxed_init,		/* tp_init */
-    (allocfunc)0,			/* tp_alloc */
-    (newfunc)0,				/* tp_new */
-    (freefunc)pyg_boxed_free,		/* tp_free */
-    (inquiry)0,				/* tp_is_gc */
-    (PyObject *)0,			/* tp_bases */
-};
-
-
 /**
  * pyg_register_boxed:
  * @dict: the module dictionary to store the wrapper class.
@@ -170,7 +130,7 @@ pyg_register_boxed(PyObject *dict, const gchar *class_name,
 
     if (!type->tp_dealloc)  type->tp_dealloc  = (destructor)pyg_boxed_dealloc;
 
-    type->ob_type = &PyType_Type;
+    Py_TYPE(type) = &PyType_Type;
     type->tp_base = &PyGBoxed_Type;
 
     if (PyType_Ready(type) < 0) {
@@ -213,11 +173,11 @@ pyg_boxed_new(GType boxed_type, gpointer boxed, gboolean copy_boxed,
     g_return_val_if_fail(boxed_type != 0, NULL);
     g_return_val_if_fail(!copy_boxed || (copy_boxed && own_ref), NULL);
 
-    state = pyg_gil_state_ensure();
+    state = pyglib_gil_state_ensure();
 
     if (!boxed) {
 	Py_INCREF(Py_None);
-	pyg_gil_state_release(state);
+	pyglib_gil_state_release(state);
 	return Py_None;
     }
 
@@ -227,7 +187,7 @@ pyg_boxed_new(GType boxed_type, gpointer boxed, gboolean copy_boxed,
     self = PyObject_NEW(PyGBoxed, tp);
 
     if (self == NULL) {
-	pyg_gil_state_release(state);
+	pyglib_gil_state_release(state);
         return NULL;
     }
 
@@ -237,8 +197,25 @@ pyg_boxed_new(GType boxed_type, gpointer boxed, gboolean copy_boxed,
     self->gtype = boxed_type;
     self->free_on_dealloc = own_ref;
 
-    pyg_gil_state_release(state);
+    pyglib_gil_state_release(state);
     
     return (PyObject *)self;
 }
 
+void
+pygobject_boxed_register_types(PyObject *d)
+{
+    pygboxed_type_key        = g_quark_from_static_string("PyGBoxed::class");
+    pygboxed_marshal_key     = g_quark_from_static_string("PyGBoxed::marshal");
+
+    PyGBoxed_Type.tp_dealloc = (destructor)pyg_boxed_dealloc;
+    PyGBoxed_Type.tp_compare = (cmpfunc)pyg_boxed_compare;
+    PyGBoxed_Type.tp_repr = (reprfunc)pyg_boxed_repr;
+    PyGBoxed_Type.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE;
+    PyGBoxed_Type.tp_methods = pygboxed_methods;
+    PyGBoxed_Type.tp_init = (initproc)pyg_boxed_init;
+    PyGBoxed_Type.tp_free = (freefunc)pyg_boxed_free;
+    PyGBoxed_Type.tp_hash = (hashfunc)pyg_boxed_hash;
+    
+    PYGOBJECT_REGISTER_GTYPE(d, PyGBoxed_Type, "GBoxed", G_TYPE_BOXED);
+}
