@@ -233,13 +233,22 @@ class TestProperty(unittest.TestCase):
         self.assertRaises(TypeError, gobject.property, type='str')
         self.assertRaises(TypeError, gobject.property, nick=False)
         self.assertRaises(TypeError, gobject.property, blurb=False)
-        self.assertRaises(TypeError, gobject.property, type=bool, default=0)
+        # this never fail while bool is a subclass of int
+        # >>> bool.__bases__
+        # (<type 'int'>,)
+        # self.assertRaises(TypeError, gobject.property, type=bool, default=0)
+        self.assertRaises(TypeError, gobject.property, type=bool, default='ciao mamma')
+        self.assertRaises(TypeError, gobject.property, type=bool)
         self.assertRaises(TypeError, gobject.property, type=GEnum)
         self.assertRaises(TypeError, gobject.property, type=GEnum, default=0)
         self.assertRaises(TypeError, gobject.property, type=object, default=0)
         self.assertRaises(TypeError, gobject.property, type=complex)
         self.assertRaises(TypeError, gobject.property, flags=-10)
 
+    def testDefaults(self):
+        p1 = gobject.property(type=bool, default=True)
+        p2 = gobject.property(type=bool, default=False)
+
     def testNameWithUnderscore(self):
         class C(gobject.GObject):
             prop_name = gobject.property(type=int)
@@ -304,3 +313,55 @@ class TestProperty(unittest.TestCase):
         pobj1 = pobj2.obj
         self.assertEqual(hash(pobj1), obj1_hash)
 
+    def testPropertySubclass(self):
+        # test for #470718
+        class A(GObject):
+            prop1 = gobject.property(type=int)
+
+        class B(A):
+            prop2 = gobject.property(type=int)
+
+        b = B()
+        b.prop2 = 10
+        self.assertEquals(b.prop2, 10)
+        b.prop1 = 20
+        self.assertEquals(b.prop1, 20)
+
+    def testPropertySubclassCustomSetter(self):
+        # test for #523352
+        class A(GObject):
+            def get_first(self):
+                return 'first'
+            first = gobject.property(type=str, getter=get_first)
+
+        class B(A):
+            def get_second(self):
+                return 'second'
+            second = gobject.property(type=str, getter=get_second)
+
+        a = A()
+        self.assertEquals(a.first, 'first')
+        self.assertRaises(TypeError, setattr, a, 'first', 'foo')
+
+        b = B()
+        self.assertEquals(b.first, 'first')
+        self.assertRaises(TypeError, setattr, b, 'first', 'foo')
+        self.assertEquals(b.second, 'second')
+        self.assertRaises(TypeError, setattr, b, 'second', 'foo')
+
+    def testPropertySubclassCustomSetterError(self):
+        try:
+            class A(GObject):
+                def get_first(self):
+                    return 'first'
+                first = gobject.property(type=str, getter=get_first)
+
+                def do_get_property(self, pspec):
+                    pass
+        except TypeError:
+            pass
+        else:
+            raise AssertionError
+
+if __name__ == '__main__':
+    unittest.main()
